import { createSignal, onMount, Show } from "solid-js";
import { Params, useParams } from "@solidjs/router";
import EmbalseType from "../interfaces/Embalse";
import Navbar from "../partials/Navbar";
import Loader from "../partials/Loader";
import Api from "../helpers/Api";
import Card from "../partials/Card";
import { FaSolidLocationDot, FaSolidMapLocationDot, FaSolidWater } from "solid-icons/fa";
import OpenWeather from "../helpers/OpenWeather";

interface CustomParams extends Params {
  id: string;
}

function Embalse() {
  const params = useParams<CustomParams>();
  const [embalse, setEmbalse] = createSignal<EmbalseType>();
  const [prediction, setPrediction] = createSignal('');

  onMount(() => {
    getEmbalse().then(async () => {
      setPrediction(await OpenWeather.week(embalse()!.x, embalse()!.y));
    });
  });

  const getEmbalse = async () => {
    const res = await Api.embalseById(params.id);

    setEmbalse(res[0]);
  }

  const getPercentage = () => {
    const perc = Math.round(embalse()!.agua_actual / embalse()!.agua_total * 100);
    return (
      <span classList={{
        'has-text-danger': perc <= 50,
        'has-text-warning': perc > 50 && perc < 70,
        'has-text-success': perc >= 70
      }}>{perc}%</span>
    )
  }

  return (
    <>
      <Navbar />
      <Show when={embalse() !== undefined} fallback={<Loader />}>
        <section class="hero is-fullheight">
          <div class="hero-body">
            <div class="container has-text-centered">
              <div class="columns is-centered">
                <div class="column is-half">
                  <p class="title">{embalse()!.nombre}</p>
                  <p class="subtitle">
                    <span class="icon-text">
                      <span class="icon">
                        <FaSolidMapLocationDot />
                      </span>
                      <span>{embalse()!.provincia}, {embalse()!.ccaa}</span>
                    </span>
                  </p>
                  <div class="columns is-centered is-vcentered is-multiline">
                    <div class="column is-narrow">
                      {/* Ubicación */}
                      <Card title="Ubicación" icon={<FaSolidLocationDot />}>
                        <>
                          <p><b>Latitud</b>: {embalse()!.x}</p>
                          <p><b>Longitud</b>: {embalse()!.y}</p>
                        </>
                      </Card>
                    </div>
                    <div class="column is-narrow">
                      <Card title="Caudal" icon={<FaSolidWater />}>
                        <>
                          <p><b>Actual</b>: {embalse()!.agua_actual} l/m2</p>
                          <p><b>Total</b>: {embalse()!.agua_total} l/m2</p>
                          <p><b>Porcentaje</b>: {getPercentage()}</p>
                        </>
                      </Card>
                    </div>
                    <div class="column is-narrow">
                      <Card title="Predicción" icon={<FaSolidCircleInfo />}>
                        <p>{prediction()}</p>
                      </Card>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
        </section>
      </Show>
    </>
  )
}

export default Embalse;
